@@ -9,6 +9,7 @@ import (
 	"strconv"
 	"strings"
 	"text/template"
+	"time"
 
 	vfs "github.com/twpayne/go-vfs/v4"
 	"go.uber.org/multierr"
@@ -64,6 +65,11 @@ func (s *ExternalDiffSystem) Chmod(name AbsPath, mode fs.FileMode) error {
 	return s.system.Chmod(name, mode)
 }
 
+// Chtimes implements System.Chtimes.
+func (s *ExternalDiffSystem) Chtimes(name AbsPath, atime, mtime time.Time) error {
+	return s.system.Chtimes(name, atime, mtime)
+}
+
 // Glob implements System.Glob.
 func (s *ExternalDiffSystem) Glob(pattern string) ([]string, error) {
 	return s.system.Glob(pattern)
@@ -164,8 +170,7 @@ func (s *ExternalDiffSystem) RunScript(scriptname RelPath, dir AbsPath, data []b
 		if err := os.MkdirAll(targetAbsPath.Dir().String(), 0o700); err != nil {
 			return err
 		}
-		//nolint:gosec
-		if err := os.WriteFile(targetAbsPath.String(), data, 0o700); err != nil {
+		if err := os.WriteFile(targetAbsPath.String(), data, 0o700); err != nil { //nolint:gosec
 			return err
 		}
 		if err := s.runDiffCommand(devNullAbsPath, targetAbsPath); err != nil {
@@ -291,8 +296,7 @@ func (s *ExternalDiffSystem) runDiffCommand(destAbsPath, targetAbsPath AbsPath)
 		args = append(args, templateData.Destination, templateData.Target)
 	}
 
-	//nolint:gosec
-	cmd := exec.Command(s.command, args...)
+	cmd := exec.Command(s.command, args...) //nolint:gosec
 	cmd.Stdin = os.Stdin
 	cmd.Stdout = os.Stdout
 	cmd.Stderr = os.Stderr
