@@ -1,14 +1,11 @@
 package chezmoi
 
 import (
-	"bytes"
-	"compress/gzip"
 	"encoding/json"
-	"io"
+	"fmt"
 	"strings"
 
 	"github.com/pelletier/go-toml/v2"
-	"go.uber.org/multierr"
 	"gopkg.in/yaml.v3"
 )
 
@@ -26,9 +23,6 @@ type Format interface {
 	Unmarshal(data []byte, value any) error
 }
 
-// A formatGzippedJSON implements the gzipped JSON serialization format.
-type formatGzippedJSON struct{}
-
 // A formatJSON implements the JSON serialization format.
 type formatJSON struct{}
 
@@ -38,50 +32,24 @@ type formatTOML struct{}
 // A formatYAML implements the YAML serialization format.
 type formatYAML struct{}
 
-// Formats is a map of all Formats by name.
-var Formats = map[string]Format{
-	"json": FormatJSON,
-	"toml": FormatTOML,
-	"yaml": FormatYAML,
-}
+var (
+	// FormatsByName is a map of all FormatsByName by name.
+	FormatsByName = map[string]Format{
+		"json": FormatJSON,
+		"toml": FormatTOML,
+		"yaml": FormatYAML,
+	}
 
-// Marshal implements Format.Marshal.
-func (formatGzippedJSON) Marshal(value any) ([]byte, error) {
-	jsonData, err := json.Marshal(value)
-	if err != nil {
-		return nil, err
+	// Formats is a map of all Formats by extension.
+	FormatsByExtension = map[string]Format{
+		"json": FormatJSON,
+		"toml": FormatTOML,
+		"yaml": FormatYAML,
+		"yml":  FormatYAML,
 	}
-	builder := &strings.Builder{}
-	builder.Grow(len(jsonData))
-	gzipWriter := gzip.NewWriter(builder)
-	if _, err := gzipWriter.Write(jsonData); err != nil {
-		return nil, err
-	}
-	if err := gzipWriter.Close(); err != nil {
-		return nil, err
-	}
-	return []byte(builder.String()), nil
-}
 
-// Name implements Format.Name.
-func (formatGzippedJSON) Name() string {
-	return "json.gz"
-}
-
-// Unmask implements Format.Unmarshal.
-func (formatGzippedJSON) Unmarshal(data []byte, value any) (err error) {
-	var r *gzip.Reader
-	if r, err = gzip.NewReader(bytes.NewReader(data)); err != nil {
-		return
-	}
-	defer multierr.AppendInvoke(&err, multierr.Close(r))
-	jsonData, err := io.ReadAll(r)
-	if err != nil {
-		return
-	}
-	err = json.Unmarshal(jsonData, value)
-	return
-}
+	FormatExtensions = sortedKeys(FormatsByExtension)
+)
 
 // Marshal implements Format.Marshal.
 func (formatJSON) Marshal(value any) ([]byte, error) {
@@ -132,9 +100,27 @@ func (formatYAML) Unmarshal(data []byte, value any) error {
 	return yaml.Unmarshal(data, value)
 }
 
+// FormatFromAbsPath returns the expected format of absPath.
+func FormatFromAbsPath(absPath AbsPath) (Format, error) {
+	format, err := formatFromExtension(absPath.Ext())
+	if err != nil {
+		return nil, fmt.Errorf("%s: %w", absPath, err)
+	}
+	return format, nil
+}
+
+// formatFromExtension returns the expected format of absPath.
+func formatFromExtension(extension string) (Format, error) {
+	format, ok := FormatsByExtension[strings.TrimPrefix(extension, ".")]
+	if !ok {
+		return nil, fmt.Errorf("%s: unknown format", extension)
+	}
+	return format, nil
+}
+
 func isPrefixDotFormat(name, prefix string) bool {
-	for _, format := range Formats {
-		if name == prefix+"."+format.Name() {
+	for extension := range FormatsByExtension {
+		if name == prefix+"."+extension {
 			return true
 		}
 	}
