@@ -70,15 +70,6 @@ type External struct {
 	sourceAbsPath   AbsPath
 }
 
-// A externalCacheEntry is an external cache entry.
-type externalCacheEntry struct {
-	URL  string    `json:"url" time:"url"`
-	Time time.Time `json:"time" yaml:"time"`
-	Data []byte    `json:"data" yaml:"data"`
-}
-
-var externalCacheFormat = formatGzippedJSON{}
-
 // A SourceState is a source state.
 type SourceState struct {
 	sync.Mutex
@@ -770,7 +761,7 @@ TARGET:
 // ReadOptions are options to SourceState.Read.
 type ReadOptions struct {
 	ReadHTTPResponse func(*http.Response) ([]byte, error)
-	RefreshExternals bool
+	RefreshExternals RefreshExternals
 	TimeNow          func() time.Time
 }
 
@@ -1021,7 +1012,7 @@ func (s *SourceState) Read(ctx context.Context, options *ReadOptions) error {
 			sourceStateCommand := &SourceStateCommand{
 				cmd:           cmd,
 				origin:        external,
-				forceRefresh:  options.RefreshExternals,
+				forceRefresh:  options.RefreshExternals == RefreshExternalsAlways,
 				refreshPeriod: external.RefreshPeriod,
 			}
 			allSourceStateEntries[externalRelPath] = append(allSourceStateEntries[externalRelPath], sourceStateCommand)
@@ -1039,7 +1030,7 @@ func (s *SourceState) Read(ctx context.Context, options *ReadOptions) error {
 			sourceStateCommand := &SourceStateCommand{
 				cmd:           cmd,
 				origin:        external,
-				forceRefresh:  options.RefreshExternals,
+				forceRefresh:  options.RefreshExternals == RefreshExternalsAlways,
 				refreshPeriod: external.RefreshPeriod,
 			}
 			allSourceStateEntries[externalRelPath] = append(allSourceStateEntries[externalRelPath], sourceStateCommand)
@@ -1133,9 +1124,9 @@ func (s *SourceState) addExternal(sourceAbsPath AbsPath) error {
 	parentSourceRelPath := NewSourceRelDirPath(parentRelPath.String())
 	parentTargetSourceRelPath := parentSourceRelPath.TargetRelPath(s.encryption.EncryptedSuffix())
 
-	format, ok := Formats[strings.TrimPrefix(sourceAbsPath.Ext(), ".")]
-	if !ok {
-		return fmt.Errorf("%s: unknown format", sourceAbsPath)
+	format, err := FormatFromAbsPath(sourceAbsPath)
+	if err != nil {
+		return err
 	}
 	data, err := s.executeTemplate(sourceAbsPath)
 	if err != nil {
@@ -1201,9 +1192,9 @@ func (s *SourceState) addPatterns(patternSet *patternSet, sourceAbsPath AbsPath,
 
 // addTemplateData adds all template data in sourceAbsPath to s.
 func (s *SourceState) addTemplateData(sourceAbsPath AbsPath) error {
-	format, ok := Formats[strings.TrimPrefix(sourceAbsPath.Ext(), ".")]
-	if !ok {
-		return fmt.Errorf("%s: unknown format", sourceAbsPath)
+	format, err := FormatFromAbsPath(sourceAbsPath)
+	if err != nil {
+		return err
 	}
 	data, err := s.system.ReadFile(sourceAbsPath)
 	if err != nil {
@@ -1287,19 +1278,30 @@ func (s *SourceState) getExternalDataRaw(
 	}
 	now = now.UTC()
 
+	refreshExternals := RefreshExternalsAuto
+	if options != nil {
+		refreshExternals = options.RefreshExternals
+	}
 	cacheKey := hex.EncodeToString(SHA256Sum([]byte(external.URL)))
-	cachedDataAbsPath := s.cacheDirAbsPath.JoinString("external", cacheKey+"."+externalCacheFormat.Name())
-	if options == nil || !options.RefreshExternals {
-		if data, err := s.system.ReadFile(cachedDataAbsPath); err == nil {
-			var externalCacheEntry externalCacheEntry
-			if err := externalCacheFormat.Unmarshal(data, &externalCacheEntry); err == nil {
-				if externalCacheEntry.URL == external.URL {
-					if external.RefreshPeriod == 0 || externalCacheEntry.Time.Add(time.Duration(external.RefreshPeriod)).After(now) {
-						return externalCacheEntry.Data, nil
-					}
+	cachedDataAbsPath := s.cacheDirAbsPath.JoinString("external", cacheKey)
+	switch refreshExternals {
+	case RefreshExternalsAlways:
+		// Never use the cache.
+	case RefreshExternalsAuto:
+		// Use the cache, if available and within the refresh period.
+		if fileInfo, err := s.baseSystem.Stat(cachedDataAbsPath); err == nil {
+			if external.RefreshPeriod == 0 || fileInfo.ModTime().Add(time.Duration(external.RefreshPeriod)).After(now) {
+				if data, err := s.baseSystem.ReadFile(cachedDataAbsPath); err == nil {
+					return data, nil
 				}
 			}
 		}
+	case RefreshExternalsNever:
+		// Always use the cache, if available, irrespective of the refresh
+		// period.
+		if data, err := s.baseSystem.ReadFile(cachedDataAbsPath); err == nil {
+			return data, nil
+		}
 	}
 
 	req, err := http.NewRequestWithContext(ctx, http.MethodGet, external.URL, nil)
@@ -1324,18 +1326,13 @@ func (s *SourceState) getExternalDataRaw(
 		return nil, fmt.Errorf("%s: %s: %s", externalRelPath, external.URL, resp.Status)
 	}
 
-	cachedExternalData, err := externalCacheFormat.Marshal(&externalCacheEntry{
-		URL:  external.URL,
-		Time: now,
-		Data: data,
-	})
-	if err != nil {
-		return nil, err
-	}
 	if err := MkdirAll(s.baseSystem, cachedDataAbsPath.Dir(), 0o700); err != nil {
 		return nil, err
 	}
-	if err := s.baseSystem.WriteFile(cachedDataAbsPath, cachedExternalData, 0o600); err != nil {
+	if err := s.baseSystem.WriteFile(cachedDataAbsPath, data, 0o600); err != nil {
+		return nil, err
+	}
+	if err := s.baseSystem.Chtimes(cachedDataAbsPath, now, now); err != nil {
 		return nil, err
 	}
 
@@ -1360,8 +1357,7 @@ func (s *SourceState) getExternalData(
 	}
 
 	if external.Filter.Command != "" {
-		//nolint:gosec
-		cmd := exec.Command(external.Filter.Command, external.Filter.Args...)
+		cmd := exec.Command(external.Filter.Command, external.Filter.Args...) //nolint:gosec
 		cmd.Stdin = bytes.NewReader(data)
 		cmd.Stderr = os.Stderr
 		data, err = chezmoilog.LogCmdOutput(cmd)
@@ -1622,12 +1618,12 @@ func (s *SourceState) newSourceStateFile(
 	case SourceFileTypeModify:
 		// If the target has an extension, determine if it indicates an
 		// interpreter to use.
-		ext := strings.ToLower(strings.TrimPrefix(targetRelPath.Ext(), "."))
-		interpreter := s.interpreters[ext]
+		extension := strings.ToLower(strings.TrimPrefix(targetRelPath.Ext(), "."))
+		interpreter := s.interpreters[extension]
 		if interpreter != nil {
 			// For modify scripts, the script extension is not considered part
 			// of the target name, so remove it.
-			targetRelPath = targetRelPath.Slice(0, targetRelPath.Len()-len(ext)-1)
+			targetRelPath = targetRelPath.Slice(0, targetRelPath.Len()-len(extension)-1)
 		}
 		targetStateEntryFunc = s.newModifyTargetStateEntryFunc(sourceRelPath, fileAttr, sourceLazyContents, interpreter)
 	case SourceFileTypeRemove:
@@ -1635,8 +1631,8 @@ func (s *SourceState) newSourceStateFile(
 	case SourceFileTypeScript:
 		// If the script has an extension, determine if it indicates an
 		// interpreter to use.
-		ext := strings.ToLower(strings.TrimPrefix(targetRelPath.Ext(), "."))
-		interpreter := s.interpreters[ext]
+		extension := strings.ToLower(strings.TrimPrefix(targetRelPath.Ext(), "."))
+		interpreter := s.interpreters[extension]
 		targetStateEntryFunc = s.newScriptTargetStateEntryFunc(
 			sourceRelPath, fileAttr, targetRelPath, sourceLazyContents, interpreter,
 		)
