@@ -7,6 +7,7 @@ import (
 	"os/exec"
 	"sort"
 	"strings"
+	"time"
 
 	vfs "github.com/twpayne/go-vfs/v4"
 	"golang.org/x/sync/errgroup"
@@ -14,10 +15,9 @@ import (
 
 // A System reads from and writes to a filesystem, runs scripts, and persists
 // state.
-//
-//nolint:interfacebloat
-type System interface {
+type System interface { //nolint:interfacebloat
 	Chmod(name AbsPath, mode fs.FileMode) error
+	Chtimes(name AbsPath, atime, mtime time.Time) error
 	Glob(pattern string) ([]string, error)
 	Link(oldname, newname AbsPath) error
 	Lstat(filename AbsPath) (fs.FileInfo, error)
@@ -56,6 +56,10 @@ func (noUpdateSystemMixin) Chmod(name AbsPath, perm fs.FileMode) error {
 	panic("update to no update system")
 }
 
+func (noUpdateSystemMixin) Chtimes(name AbsPath, atime, mtime time.Time) error {
+	panic("update to no update system")
+}
+
 func (noUpdateSystemMixin) Link(oldname, newname AbsPath) error {
 	panic("update to no update system")
 }
