@@ -4,6 +4,7 @@ import (
 	"archive/zip"
 	"io"
 	"io/fs"
+	"os/exec"
 	"time"
 )
 
@@ -40,6 +41,11 @@ func (s *ZIPWriterSystem) Mkdir(name AbsPath, perm fs.FileMode) error {
 	return err
 }
 
+// RunCmd implements System.RunCmd.
+func (s *ZIPWriterSystem) RunCmd(cmd *exec.Cmd) error {
+	return nil
+}
+
 // RunScript implements System.RunScript.
 func (s *ZIPWriterSystem) RunScript(scriptname RelPath, dir AbsPath, data []byte, interpreter *Interpreter) error {
 	return s.WriteFile(NewAbsPath(scriptname.String()), data, 0o700)
