@@ -29,15 +29,14 @@ import (
 	"github.com/coreos/go-semver/semver"
 	gogit "github.com/go-git/go-git/v5"
 	"github.com/go-git/go-git/v5/plumbing/format/diff"
-	"github.com/google/gops/agent"
 	"github.com/gregjones/httpcache"
 	"github.com/gregjones/httpcache/diskcache"
 	"github.com/mitchellh/mapstructure"
+	"github.com/muesli/termenv"
 	"github.com/rs/zerolog"
 	"github.com/rs/zerolog/log"
-	"github.com/spf13/afero"
 	"github.com/spf13/cobra"
-	"github.com/spf13/viper"
+	"github.com/spf13/pflag"
 	"github.com/twpayne/go-shell"
 	"github.com/twpayne/go-vfs/v4"
 	"github.com/twpayne/go-xdg/v6"
@@ -69,6 +68,10 @@ type templateConfig struct {
 	Options []string `mapstructure:"options"`
 }
 
+type warningsConfig struct {
+	ConfigFileTemplateHasChanged bool `mapstructure:"configFileTemplateHasChanged"`
+}
+
 // ConfigFile contains all data settable in the config file.
 type ConfigFile struct {
 	// Global configuration.
@@ -91,6 +94,7 @@ type ConfigFile struct {
 	UseBuiltinAge      autoBool                        `mapstructure:"useBuiltinAge"`
 	UseBuiltinGit      autoBool                        `mapstructure:"useBuiltinGit"`
 	Verbose            bool                            `mapstructure:"verbose"`
+	Warnings           warningsConfig                  `mapstructure:"warnings"`
 	WorkingTreeAbsPath chezmoi.AbsPath                 `mapstructure:"workingTree"`
 
 	// Password manager configurations.
@@ -133,14 +137,13 @@ type Config struct {
 	debug            bool
 	dryRun           bool
 	force            bool
-	gops             bool
 	homeDir          string
 	interactive      bool
 	keepGoing        bool
 	noPager          bool
 	noTTY            bool
 	outputAbsPath    chezmoi.AbsPath
-	refreshExternals bool
+	refreshExternals chezmoi.RefreshExternals
 	sourcePath       bool
 	templateFuncs    template.FuncMap
 
@@ -185,6 +188,8 @@ type Config struct {
 	// Computed configuration.
 	homeDirAbsPath chezmoi.AbsPath
 	encryption     chezmoi.Encryption
+	sourceState    *chezmoi.SourceState
+	sourceStateErr error
 
 	stdin       io.Reader
 	stdout      io.Writer
@@ -222,18 +227,6 @@ var (
 	}
 
 	whitespaceRx = regexp.MustCompile(`\s+`)
-
-	viperDecodeConfigOptions = []viper.DecoderConfigOption{
-		viper.DecodeHook(
-			mapstructure.ComposeDecodeHookFunc(
-				mapstructure.StringToTimeDurationHookFunc(),
-				mapstructure.StringToSliceHookFunc(","),
-				chezmoi.StringSliceToEntryTypeSetHookFunc(),
-				chezmoi.StringToAbsPathHookFunc(),
-				StringOrBoolToAutoBoolHookFunc(),
-			),
-		),
-	}
 )
 
 // newConfig creates a new Config with the given options.
@@ -252,113 +245,14 @@ func newConfig(options ...configOption) (*Config, error) {
 		return nil, err
 	}
 
-	cacheDirAbsPath := chezmoi.NewAbsPath(bds.CacheHome).Join(chezmoiRelPath)
-
-	configFile := ConfigFile{
-		// Global configuration.
-		CacheDirAbsPath: cacheDirAbsPath,
-		Color: autoBool{
-			auto: true,
-		},
-		Interpreters: defaultInterpreters,
-		Pager:        os.Getenv("PAGER"),
-		PINEntry: pinEntryConfig{
-			Options: pinEntryDefaultOptions,
-		},
-		Safe: true,
-		Template: templateConfig{
-			Options: chezmoi.DefaultTemplateOptions,
-		},
-		Umask: chezmoi.Umask,
-		UseBuiltinAge: autoBool{
-			auto: true,
-		},
-		UseBuiltinGit: autoBool{
-			auto: true,
-		},
-
-		// Password manager configurations.
-		Bitwarden: bitwardenConfig{
-			Command: "bw",
-		},
-		Gopass: gopassConfig{
-			Command: "gopass",
-		},
-		Keepassxc: keepassxcConfig{
-			Command: "keepassxc-cli",
-		},
-		Keeper: keeperConfig{
-			Command: "keeper",
-		},
-		Lastpass: lastpassConfig{
-			Command: "lpass",
-		},
-		Onepassword: onepasswordConfig{
-			Command: "op",
-			Prompt:  true,
-		},
-		Pass: passConfig{
-			Command: "pass",
-		},
-		Passhole: passholeConfig{
-			Command: "ph",
-			Prompt:  true,
-		},
-		Vault: vaultConfig{
-			Command: "vault",
-		},
-
-		// Encryption configurations.
-		Age: defaultAgeEncryptionConfig,
-		GPG: defaultGPGEncryptionConfig,
-
-		// Command configurations.
-		Add: addCmdConfig{
-			exclude:   chezmoi.NewEntryTypeSet(chezmoi.EntryTypesNone),
-			include:   chezmoi.NewEntryTypeSet(chezmoi.EntryTypesAll),
-			recursive: true,
-		},
-		Diff: diffCmdConfig{
-			Exclude: chezmoi.NewEntryTypeSet(chezmoi.EntryTypesNone),
-			include: chezmoi.NewEntryTypeSet(chezmoi.EntryTypesAll),
-		},
-		Edit: editCmdConfig{
-			Hardlink:    true,
-			MinDuration: 1 * time.Second,
-			exclude:     chezmoi.NewEntryTypeSet(chezmoi.EntryTypesNone),
-			include: chezmoi.NewEntryTypeSet(
-				chezmoi.EntryTypeDirs | chezmoi.EntryTypeFiles | chezmoi.EntryTypeSymlinks | chezmoi.EntryTypeEncrypted,
-			),
-		},
-		Format: defaultWriteDataFormat,
-		Git: gitCmdConfig{
-			Command: "git",
-		},
-		Merge: mergeCmdConfig{
-			Command: "vimdiff",
-		},
-		Status: statusCmdConfig{
-			Exclude:   chezmoi.NewEntryTypeSet(chezmoi.EntryTypesNone),
-			include:   chezmoi.NewEntryTypeSet(chezmoi.EntryTypesAll),
-			recursive: true,
-		},
-		Verify: verifyCmdConfig{
-			Exclude:   chezmoi.NewEntryTypeSet(chezmoi.EntryTypesNone),
-			include:   chezmoi.NewEntryTypeSet(chezmoi.EntryTypesAll &^ chezmoi.EntryTypeScripts),
-			recursive: true,
-		},
-	}
-
 	c := &Config{
-		ConfigFile: configFile,
+		ConfigFile: newConfigFile(bds),
 
 		// Global configuration.
 		homeDir:       userHomeDir,
 		templateFuncs: sprig.TxtFuncMap(),
 
-		// Password manager data.
-
-		// Command configurations, not settable in the config file.
+		// Command configurations.
 		apply: applyCmdConfig{
 			exclude:   chezmoi.NewEntryTypeSet(chezmoi.EntryTypesNone),
 			include:   chezmoi.NewEntryTypeSet(chezmoi.EntryTypesAll),
@@ -516,7 +410,6 @@ func (c *Config) Close() error {
 		err = multierr.Append(err, err2)
 	}
 	pprof.StopCPUProfile()
-	agent.Close()
 	return err
 }
 
@@ -554,12 +447,12 @@ func (c *Config) applyArgs(
 	}
 
 	var currentConfigTemplateContentsSHA256 []byte
-	configTemplateRelPath, _, configTemplateContents, err := c.findFirstConfigTemplate()
+	configTemplate, err := c.findFirstConfigTemplate()
 	if err != nil {
 		return err
 	}
-	if configTemplateRelPath != chezmoi.EmptyRelPath {
-		currentConfigTemplateContentsSHA256 = chezmoi.SHA256Sum(configTemplateContents)
+	if configTemplate != nil {
+		currentConfigTemplateContentsSHA256 = chezmoi.SHA256Sum(configTemplate.contents)
 	}
 	var previousConfigTemplateContentsSHA256 []byte
 	if configStateData, err := c.persistentState.Get(chezmoi.ConfigStateBucket, configStateKey); err != nil {
@@ -576,7 +469,7 @@ func (c *Config) applyArgs(
 		bytes.Equal(currentConfigTemplateContentsSHA256, previousConfigTemplateContentsSHA256)
 	if !configTemplateContentsUnchanged {
 		if c.force {
-			if configTemplateRelPath == chezmoi.EmptyRelPath {
+			if configTemplate == nil {
 				if err := c.persistentState.Delete(chezmoi.ConfigStateBucket, configStateKey); err != nil {
 					return err
 				}
@@ -591,12 +484,12 @@ func (c *Config) applyArgs(
 					return err
 				}
 			}
-		} else {
+		} else if c.Warnings.ConfigFileTemplateHasChanged {
 			c.errorf("warning: config file template has changed, run chezmoi init to regenerate config file\n")
 		}
 	}
 
-	sourceState, err := c.newSourceState(ctx)
+	sourceState, err := c.getSourceState(ctx)
 	if err != nil {
 		return err
 	}
@@ -709,66 +602,57 @@ func (c *Config) colorAutoFunc() bool {
 // template and reloads it.
 func (c *Config) createAndReloadConfigFile() error {
 	// Find config template, execute it, and create config file.
-	configTemplateRelPath, ext, configTemplateContents, err := c.findFirstConfigTemplate()
+	configTemplate, err := c.findFirstConfigTemplate()
 	if err != nil {
 		return err
 	}
-	var configFileContents []byte
-	if configTemplateRelPath == chezmoi.EmptyRelPath {
+
+	if configTemplate == nil {
 		if err := c.persistentState.Delete(chezmoi.ConfigStateBucket, configStateKey); err != nil {
 			return err
 		}
-	} else {
-		configFileContents, err = c.createConfigFile(configTemplateRelPath, configTemplateContents)
-		if err != nil {
-			return err
-		}
-
-		// Validate the config.
-		v := viper.New()
-		v.SetConfigType(ext)
-		if err := v.ReadConfig(bytes.NewBuffer(configFileContents)); err != nil {
-			return err
-		}
-		if err := v.Unmarshal(&Config{}, viperDecodeConfigOptions...); err != nil {
-			return err
-		}
-
-		// Write the config.
-		configPath := c.init.configPath
-		if c.init.configPath.Empty() {
-			configPath = chezmoi.NewAbsPath(c.bds.ConfigHome).Join(chezmoiRelPath, configTemplateRelPath)
-		}
-		if err := chezmoi.MkdirAll(c.baseSystem, configPath.Dir(), 0o777); err != nil {
-			return err
-		}
-		if err := c.baseSystem.WriteFile(configPath, configFileContents, 0o600); err != nil {
-			return err
-		}
-
-		configStateValue, err := json.Marshal(configState{
-			ConfigTemplateContentsSHA256: chezmoi.HexBytes(chezmoi.SHA256Sum(configTemplateContents)),
-		})
-		if err != nil {
-			return err
-		}
-		if err := c.persistentState.Set(chezmoi.ConfigStateBucket, configStateKey, configStateValue); err != nil {
-			return err
-		}
+		return nil
 	}
 
-	// Reload config if it was created.
-	if configTemplateRelPath != chezmoi.EmptyRelPath {
-		viper.SetConfigType(ext)
-		if err := viper.ReadConfig(bytes.NewBuffer(configFileContents)); err != nil {
-			return err
-		}
-		if err := viper.Unmarshal(&c.ConfigFile, viperDecodeConfigOptions...); err != nil {
-			return err
-		}
-		if err := c.setEncryption(); err != nil {
-			return err
-		}
+	configFileContents, err := c.createConfigFile(configTemplate.targetRelPath, configTemplate.contents)
+	if err != nil {
+		return err
+	}
+
+	// Validate the configMap.
+	var configFile ConfigFile
+	if err := c.decodeConfigBytes(configTemplate.format, configFileContents, &configFile); err != nil {
+		return fmt.Errorf("%s: %w", configTemplate.sourceAbsPath, err)
+	}
+
+	// Write the config.
+	configPath := c.init.configPath
+	if c.init.configPath.Empty() {
+		configPath = chezmoi.NewAbsPath(c.bds.ConfigHome).Join(chezmoiRelPath, configTemplate.targetRelPath)
+	}
+	if err := chezmoi.MkdirAll(c.baseSystem, configPath.Dir(), 0o777); err != nil {
+		return err
+	}
+	if err := c.baseSystem.WriteFile(configPath, configFileContents, 0o600); err != nil {
+		return err
+	}
+
+	configStateValue, err := json.Marshal(configState{
+		ConfigTemplateContentsSHA256: chezmoi.HexBytes(chezmoi.SHA256Sum(configTemplate.contents)),
+	})
+	if err != nil {
+		return err
+	}
+	if err := c.persistentState.Set(chezmoi.ConfigStateBucket, configStateKey, configStateValue); err != nil {
+		return err
+	}
+
+	// Reload the config.
+	if err := c.decodeConfigBytes(configTemplate.format, configFileContents, &c.ConfigFile); err != nil {
+		return fmt.Errorf("%s: %w", configTemplate.sourceAbsPath, err)
+	}
+	if err := c.setEncryption(); err != nil {
+		return err
 	}
 
 	return nil
@@ -819,7 +703,7 @@ func (c *Config) defaultConfigFile(
 		if err != nil {
 			return chezmoi.EmptyAbsPath, err
 		}
-		for _, extension := range viper.SupportedExts {
+		for _, extension := range chezmoi.FormatExtensions {
 			configFileAbsPath := configDirAbsPath.JoinString("chezmoi", "chezmoi."+extension)
 			if _, err := fileSystem.Stat(configFileAbsPath.String()); err == nil {
 				return configFileAbsPath, nil
@@ -834,6 +718,62 @@ func (c *Config) defaultConfigFile(
 	return configHomeAbsPath.JoinString("chezmoi", "chezmoi.toml"), nil
 }
 
+// decodeConfigBytes decodes data in format into configFile.
+func (c *Config) decodeConfigBytes(format chezmoi.Format, data []byte, configFile *ConfigFile) error {
+	var configMap map[string]any
+	if err := format.Unmarshal(data, &configMap); err != nil {
+		return err
+	}
+	return c.decodeConfigMap(configMap, configFile)
+}
+
+// decodeConfigFile decodes the config file at configFileAbsPath into
+// configFile.
+func (c *Config) decodeConfigFile(configFileAbsPath chezmoi.AbsPath, configFile *ConfigFile) error {
+	var format chezmoi.Format
+	if c.configFormat == "" {
+		var err error
+		format, err = chezmoi.FormatFromAbsPath(configFileAbsPath)
+		if err != nil {
+			return err
+		}
+	} else {
+		format = c.configFormat.Format()
+	}
+
+	configFileContents, err := c.fileSystem.ReadFile(configFileAbsPath.String())
+	if err != nil {
+		return fmt.Errorf("%s: %w", configFileAbsPath, err)
+	}
+
+	if err := c.decodeConfigBytes(format, configFileContents, configFile); err != nil {
+		return fmt.Errorf("%s: %w", configFileAbsPath, err)
+	}
+
+	return nil
+}
+
+// decodeConfigMap decodes configMap into configFile.
+func (c *Config) decodeConfigMap(configMap map[string]any, configFile *ConfigFile) error {
+	decoder, err := mapstructure.NewDecoder(&mapstructure.DecoderConfig{
+		DecodeHook: mapstructure.ComposeDecodeHookFunc(
+			mapstructure.StringToTimeDurationHookFunc(),
+			mapstructure.StringToSliceHookFunc(","),
+			chezmoi.StringSliceToEntryTypeSetHookFunc(),
+			chezmoi.StringToAbsPathHookFunc(),
+			StringOrBoolToAutoBoolHookFunc(),
+		),
+		Result: configFile,
+	})
+	if err != nil {
+		return err
+	}
+	if err := decoder.Decode(configMap); err != nil {
+		return err
+	}
+	return nil
+}
+
 // defaultPreApplyFunc is the default pre-apply function. If the target entry
 // has changed since chezmoi last wrote it then it prompts the user for the
 // action to take.
@@ -1046,7 +986,7 @@ func (c *Config) defaultTemplateData() map[string]any {
 
 	executable, _ := os.Executable()
 
-	windowsVersion, _ := c.windowsVersion()
+	windowsVersion, _ := windowsVersion()
 
 	return map[string]any{
 		"chezmoi": map[string]any{
@@ -1234,26 +1174,39 @@ func (c *Config) filterInput(args []string, f func([]byte) ([]byte, error)) erro
 	return nil
 }
 
+type configTemplate struct {
+	sourceAbsPath chezmoi.AbsPath
+	format        chezmoi.Format
+	targetRelPath chezmoi.RelPath
+	contents      []byte
+}
+
 // findFirstConfigTemplate searches for a config template, returning the path,
 // format, and contents of the first one that it finds.
-func (c *Config) findFirstConfigTemplate() (chezmoi.RelPath, string, []byte, error) {
+func (c *Config) findFirstConfigTemplate() (*configTemplate, error) {
 	sourceDirAbsPath, err := c.sourceDirAbsPath()
 	if err != nil {
-		return chezmoi.EmptyRelPath, "", nil, err
+		return nil, err
 	}
 
-	for _, ext := range viper.SupportedExts {
-		filename := chezmoi.NewRelPath(chezmoi.Prefix + "." + ext + chezmoi.TemplateSuffix)
-		contents, err := c.baseSystem.ReadFile(sourceDirAbsPath.Join(filename))
+	for _, extension := range chezmoi.FormatExtensions {
+		relPath := chezmoi.NewRelPath(chezmoi.Prefix + "." + extension + chezmoi.TemplateSuffix)
+		absPath := sourceDirAbsPath.Join(relPath)
+		contents, err := c.baseSystem.ReadFile(absPath)
 		switch {
 		case errors.Is(err, fs.ErrNotExist):
 			continue
 		case err != nil:
-			return chezmoi.EmptyRelPath, "", nil, err
+			return nil, err
 		}
-		return chezmoi.NewRelPath("chezmoi." + ext), ext, contents, nil
+		return &configTemplate{
+			targetRelPath: chezmoi.NewRelPath("chezmoi." + extension),
+			format:        chezmoi.FormatsByExtension[extension],
+			sourceAbsPath: absPath,
+			contents:      contents,
+		}, nil
 	}
-	return chezmoi.EmptyRelPath, "", nil, nil
+	return nil, nil
 }
 
 func (c *Config) getHTTPClient() (*http.Client, error) {
@@ -1272,6 +1225,14 @@ func (c *Config) getHTTPClient() (*http.Client, error) {
 	return c.httpClient, nil
 }
 
+func (c *Config) getSourceState(ctx context.Context) (*chezmoi.SourceState, error) {
+	if c.sourceState != nil || c.sourceStateErr != nil {
+		return c.sourceState, c.sourceStateErr
+	}
+	c.sourceState, c.sourceStateErr = c.newSourceState(ctx)
+	return c.sourceState, c.sourceStateErr
+}
+
 // gitAutoAdd adds all changes to the git index and returns the new git status.
 func (c *Config) gitAutoAdd() (*git.Status, error) {
 	if err := c.run(c.WorkingTreeAbsPath, c.Git.Command, []string{"add", "."}); err != nil {
@@ -1319,7 +1280,7 @@ func (c *Config) makeRunEWithSourceState(
 	runE func(*cobra.Command, []string, *chezmoi.SourceState) error,
 ) func(*cobra.Command, []string) error {
 	return func(cmd *cobra.Command, args []string) error {
-		sourceState, err := c.newSourceState(cmd.Context())
+		sourceState, err := c.getSourceState(cmd.Context())
 		if err != nil {
 			return err
 		}
@@ -1329,16 +1290,7 @@ func (c *Config) makeRunEWithSourceState(
 
 // marshal formats data in dataFormat and writes it to the standard output.
 func (c *Config) marshal(dataFormat writeDataFormat, data any) error {
-	var format chezmoi.Format
-	switch dataFormat {
-	case writeDataFormatJSON:
-		format = chezmoi.FormatJSON
-	case writeDataFormatYAML:
-		format = chezmoi.FormatYAML
-	default:
-		return fmt.Errorf("%s: unknown format", dataFormat)
-	}
-	marshaledData, err := format.Marshal(data)
+	marshaledData, err := dataFormat.Format().Marshal(data)
 	if err != nil {
 		return err
 	}
@@ -1371,24 +1323,6 @@ func (c *Config) newRootCmd() (*cobra.Command, error) {
 	persistentFlags.Var(&c.UseBuiltinGit, "use-builtin-git", "Use builtin git")
 	persistentFlags.BoolVarP(&c.Verbose, "verbose", "v", c.Verbose, "Make output more verbose")
 	persistentFlags.VarP(&c.WorkingTreeAbsPath, "working-tree", "W", "Set working tree directory")
-	for viperKey, key := range map[string]string{
-		"cacheDir":        "cache",
-		"color":           "color",
-		"destDir":         "destination",
-		"persistentState": "persistent-state",
-		"progress":        "progress",
-		"mode":            "mode",
-		"safe":            "safe",
-		"sourceDir":       "source",
-		"useBuiltinAge":   "use-builtin-age",
-		"useBuiltinGit":   "use-builtin-git",
-		"verbose":         "verbose",
-		"workingTree":     "working-tree",
-	} {
-		if err := viper.BindPFlag(viperKey, persistentFlags.Lookup(key)); err != nil {
-			return nil, err
-		}
-	}
 
 	persistentFlags.VarP(&c.configFileAbsPath, "config", "c", "Set config file")
 	persistentFlags.Var(&c.configFormat, "config-format", "Set config file format")
@@ -1396,13 +1330,13 @@ func (c *Config) newRootCmd() (*cobra.Command, error) {
 	persistentFlags.BoolVar(&c.debug, "debug", c.debug, "Include debug information in output")
 	persistentFlags.BoolVarP(&c.dryRun, "dry-run", "n", c.dryRun, "Do not make any modifications to the destination directory") //nolint:lll
 	persistentFlags.BoolVar(&c.force, "force", c.force, "Make all changes without prompting")
-	persistentFlags.BoolVar(&c.gops, "gops", c.gops, "Enable gops agent")
 	persistentFlags.BoolVar(&c.interactive, "interactive", c.interactive, "Prompt for all changes")
 	persistentFlags.BoolVarP(&c.keepGoing, "keep-going", "k", c.keepGoing, "Keep going as far as possible after an error")
 	persistentFlags.BoolVar(&c.noPager, "no-pager", c.noPager, "Do not use the pager")
 	persistentFlags.BoolVar(&c.noTTY, "no-tty", c.noTTY, "Do not attempt to get a TTY for reading passwords")
 	persistentFlags.VarP(&c.outputAbsPath, "output", "o", "Write output to path instead of stdout")
-	persistentFlags.BoolVarP(&c.refreshExternals, "refresh-externals", "R", c.refreshExternals, "Refresh external cache")
+	persistentFlags.VarP(&c.refreshExternals, "refresh-externals", "R", "Refresh external cache")
+	persistentFlags.Lookup("refresh-externals").NoOptDefVal = chezmoi.RefreshExternalsAlways.String()
 	persistentFlags.BoolVar(&c.sourcePath, "source-path", c.sourcePath, "Specify targets by source path")
 
 	for _, err := range []error{
@@ -1410,13 +1344,13 @@ func (c *Config) newRootCmd() (*cobra.Command, error) {
 		rootCmd.MarkPersistentFlagFilename("cpu-profile"),
 		persistentFlags.MarkHidden("cpu-profile"),
 		rootCmd.MarkPersistentFlagDirname("destination"),
-		persistentFlags.MarkHidden("gops"),
 		rootCmd.MarkPersistentFlagFilename("output"),
 		persistentFlags.MarkHidden("safe"),
 		rootCmd.MarkPersistentFlagDirname("source"),
 		rootCmd.RegisterFlagCompletionFunc("color", autoBoolFlagCompletionFunc),
 		rootCmd.RegisterFlagCompletionFunc("config-format", readDataFormatFlagCompletionFunc),
 		rootCmd.RegisterFlagCompletionFunc("mode", chezmoi.ModeFlagCompletionFunc),
+		rootCmd.RegisterFlagCompletionFunc("refresh-externals", chezmoi.RefreshExternalsFlagCompletionFunc),
 		rootCmd.RegisterFlagCompletionFunc("use-builtin-age", autoBoolFlagCompletionFunc),
 		rootCmd.RegisterFlagCompletionFunc("use-builtin-git", autoBoolFlagCompletionFunc),
 	} {
@@ -1515,7 +1449,7 @@ func (c *Config) newSourceState(
 		return nil, err
 	}
 
-	s := chezmoi.NewSourceState(append([]chezmoi.SourceStateOption{
+	sourceState := chezmoi.NewSourceState(append([]chezmoi.SourceStateOption{
 		chezmoi.WithBaseSystem(c.baseSystem),
 		chezmoi.WithCacheDir(c.CacheDirAbsPath),
 		chezmoi.WithDefaultTemplateDataFunc(c.defaultTemplateData),
@@ -1533,14 +1467,14 @@ func (c *Config) newSourceState(
 		chezmoi.WithVersion(c.version),
 	}, options...)...)
 
-	if err := s.Read(ctx, &chezmoi.ReadOptions{
+	if err := sourceState.Read(ctx, &chezmoi.ReadOptions{
 		RefreshExternals: c.refreshExternals,
 		ReadHTTPResponse: c.readHTTPResponse,
 	}); err != nil {
 		return nil, err
 	}
 
-	return s, nil
+	return sourceState, nil
 }
 
 // persistentPostRunRootE performs post-run actions for the root command.
@@ -1550,13 +1484,22 @@ func (c *Config) persistentPostRunRootE(cmd *cobra.Command, args []string) error
 	}
 
 	if boolAnnotation(cmd, modifiesConfigFile) {
-		// Warn the user of any errors reading the config file.
-		v := viper.New()
-		v.SetFs(afero.FromIOFS{FS: c.fileSystem})
-		v.SetConfigFile(c.configFileAbsPath.String())
-		err := v.ReadInConfig()
-		if err == nil {
-			err = v.Unmarshal(&Config{}, viperDecodeConfigOptions...)
+		configFileContents, err := c.baseSystem.ReadFile(c.configFileAbsPath)
+		switch {
+		case errors.Is(err, fs.ErrNotExist):
+			err = nil
+		case err != nil:
+			// err is already set, do nothing.
+		default:
+			var format chezmoi.Format
+			if format, err = chezmoi.FormatFromAbsPath(c.configFileAbsPath); err == nil {
+				var config map[string]any
+				if err = format.Unmarshal(configFileContents, &config); err != nil {
+					// err is already set, do nothing.
+				} else {
+					err = c.decodeConfigMap(config, &ConfigFile{})
+				}
+			}
 		}
 		if err != nil {
 			c.errorf("warning: %s: %v\n", c.configFileAbsPath, err)
@@ -1633,21 +1576,28 @@ func (c *Config) persistentPreRunRootE(cmd *cobra.Command, args []string) error
 		}
 	}
 
-	// Enable gops if configured.
-	if c.gops {
-		if err := agent.Listen(agent.Options{}); err != nil {
-			return err
-		}
-	}
-
 	if runtime.GOOS == "windows" {
 		var err error
-		c.restoreWindowsConsole, err = enableVirtualTerminalProcessing(c.stdout)
+		c.restoreWindowsConsole, err = termenv.EnableVirtualTerminalProcessing(termenv.DefaultOutput())
 		if err != nil {
 			return err
 		}
 	}
 
+	// Save flags that were set on the command line. Skip some types as
+	// spf13/pflag does not round trip them correctly.
+	changedFlags := make(map[pflag.Value]string)
+	brokenFlagTypes := map[string]bool{
+		"stringToInt":    true,
+		"stringToInt64":  true,
+		"stringToString": true,
+	}
+	cmd.Flags().VisitAll(func(flag *pflag.Flag) {
+		if flag.Changed && !brokenFlagTypes[flag.Value.Type()] {
+			changedFlags[flag.Value] = flag.Value.String()
+		}
+	})
+
 	// Read the config file.
 	if err := c.readConfig(); err != nil {
 		if !boolAnnotation(cmd, doesNotRequireValidConfig) {
@@ -1656,6 +1606,13 @@ func (c *Config) persistentPreRunRootE(cmd *cobra.Command, args []string) error
 		c.errorf("warning: %s: %v\n", c.configFileAbsPath, err)
 	}
 
+	// Restore flags that were set on the command line.
+	for value, original := range changedFlags {
+		if err := value.Set(original); err != nil {
+			return err
+		}
+	}
+
 	// Configure the logger.
 	log.Logger = log.Output(zerolog.NewConsoleWriter(
 		func(w *zerolog.ConsoleWriter) {
@@ -1866,21 +1823,12 @@ func (c *Config) persistentStateFile() (chezmoi.AbsPath, error) {
 
 // readConfig reads the config file, if it exists.
 func (c *Config) readConfig() error {
-	viper.SetConfigFile(c.configFileAbsPath.String())
-	if c.configFormat != "" {
-		viper.SetConfigType(c.configFormat.String())
-	}
-	viper.SetFs(afero.FromIOFS{FS: c.fileSystem})
-	switch err := viper.ReadInConfig(); {
+	switch err := c.decodeConfigFile(c.configFileAbsPath, &c.ConfigFile); {
 	case errors.Is(err, fs.ErrNotExist):
 		return nil
-	case err != nil:
+	default:
 		return err
 	}
-	if err := viper.Unmarshal(&c.ConfigFile, viperDecodeConfigOptions...); err != nil {
-		return err
-	}
-	return nil
 }
 
 // run runs name with args in dir.
@@ -2094,7 +2042,7 @@ func (c *Config) targetValidArgs(
 		return nil, cobra.ShellCompDirectiveError
 	}
 
-	sourceState, err := c.newSourceState(cmd.Context())
+	sourceState, err := c.getSourceState(cmd.Context())
 	if err != nil {
 		cobra.CompErrorln(err.Error())
 		return nil, cobra.ShellCompDirectiveError
@@ -2179,8 +2127,7 @@ func (c *Config) writeOutput(data []byte) error {
 		_, err := c.stdout.Write(data)
 		return err
 	}
-	//nolint:gosec
-	return os.WriteFile(c.outputAbsPath.String(), data, 0o666)
+	return os.WriteFile(c.outputAbsPath.String(), data, 0o666) //nolint:gosec
 }
 
 // writeOutputString writes data to the configured output.
@@ -2188,6 +2135,106 @@ func (c *Config) writeOutputString(data string) error {
 	return c.writeOutput([]byte(data))
 }
 
+func newConfigFile(bds *xdg.BaseDirectorySpecification) ConfigFile {
+	return ConfigFile{
+		// Global configuration.
+		CacheDirAbsPath: chezmoi.NewAbsPath(bds.CacheHome).Join(chezmoiRelPath),
+		Color: autoBool{
+			auto: true,
+		},
+		Interpreters: defaultInterpreters,
+		Pager:        os.Getenv("PAGER"),
+		PINEntry: pinEntryConfig{
+			Options: pinEntryDefaultOptions,
+		},
+		Safe: true,
+		Template: templateConfig{
+			Options: chezmoi.DefaultTemplateOptions,
+		},
+		Umask: chezmoi.Umask,
+		UseBuiltinAge: autoBool{
+			auto: true,
+		},
+		UseBuiltinGit: autoBool{
+			auto: true,
+		},
+		Warnings: warningsConfig{
+			ConfigFileTemplateHasChanged: true,
+		},
+
+		// Password manager configurations.
+		Bitwarden: bitwardenConfig{
+			Command: "bw",
+		},
+		Gopass: gopassConfig{
+			Command: "gopass",
+		},
+		Keepassxc: keepassxcConfig{
+			Command: "keepassxc-cli",
+		},
+		Keeper: keeperConfig{
+			Command: "keeper",
+		},
+		Lastpass: lastpassConfig{
+			Command: "lpass",
+		},
+		Onepassword: onepasswordConfig{
+			Command: "op",
+			Prompt:  true,
+		},
+		Pass: passConfig{
+			Command: "pass",
+		},
+		Passhole: passholeConfig{
+			Command: "ph",
+			Prompt:  true,
+		},
+		Vault: vaultConfig{
+			Command: "vault",
+		},
+
+		// Encryption configurations.
+		Age: defaultAgeEncryptionConfig,
+		GPG: defaultGPGEncryptionConfig,
+
+		// Command configurations.
+		Add: addCmdConfig{
+			exclude:   chezmoi.NewEntryTypeSet(chezmoi.EntryTypesNone),
+			include:   chezmoi.NewEntryTypeSet(chezmoi.EntryTypesAll),
+			recursive: true,
+		},
+		Diff: diffCmdConfig{
+			Exclude: chezmoi.NewEntryTypeSet(chezmoi.EntryTypesNone),
+			include: chezmoi.NewEntryTypeSet(chezmoi.EntryTypesAll),
+		},
+		Edit: editCmdConfig{
+			Hardlink:    true,
+			MinDuration: 1 * time.Second,
+			exclude:     chezmoi.NewEntryTypeSet(chezmoi.EntryTypesNone),
+			include: chezmoi.NewEntryTypeSet(
+				chezmoi.EntryTypeDirs | chezmoi.EntryTypeFiles | chezmoi.EntryTypeSymlinks | chezmoi.EntryTypeEncrypted,
+			),
+		},
+		Format: defaultWriteDataFormat,
+		Git: gitCmdConfig{
+			Command: "git",
+		},
+		Merge: mergeCmdConfig{
+			Command: "vimdiff",
+		},
+		Status: statusCmdConfig{
+			Exclude:   chezmoi.NewEntryTypeSet(chezmoi.EntryTypesNone),
+			include:   chezmoi.NewEntryTypeSet(chezmoi.EntryTypesAll),
+			recursive: true,
+		},
+		Verify: verifyCmdConfig{
+			Exclude:   chezmoi.NewEntryTypeSet(chezmoi.EntryTypesNone),
+			include:   chezmoi.NewEntryTypeSet(chezmoi.EntryTypesAll &^ chezmoi.EntryTypeScripts),
+			recursive: true,
+		},
+	}
+}
+
 func parseCommand(command string, args []string) (string, []string, error) {
 	// If command is found, then return it.
 	if path, err := chezmoi.LookPath(command); err == nil {
