@@ -21,7 +21,6 @@ import (
 	"github.com/coreos/go-semver/semver"
 	"github.com/google/go-github/v45/github"
 	"github.com/spf13/cobra"
-	"github.com/spf13/viper"
 	"github.com/twpayne/go-shell"
 	"github.com/twpayne/go-xdg/v6"
 
@@ -436,8 +435,7 @@ func (c *binaryCheck) Run(system chezmoi.System, homeDirAbsPath chezmoi.AbsPath)
 		return checkResultOK, fmt.Sprintf("found %s", pathAbsPath)
 	}
 
-	//nolint:gosec
-	cmd := exec.Command(pathAbsPath.String(), c.versionArgs...)
+	cmd := exec.Command(pathAbsPath.String(), c.versionArgs...) //nolint:gosec
 	output, err := chezmoilog.LogCmdCombinedOutput(cmd)
 	if err != nil {
 		return checkResultFailed, err.Error()
@@ -475,7 +473,7 @@ func (c *configFileCheck) Run(system chezmoi.System, homeDirAbsPath chezmoi.AbsP
 		if err != nil {
 			return checkResultFailed, err.Error()
 		}
-		for _, extension := range viper.SupportedExts {
+		for _, extension := range chezmoi.FormatExtensions {
 			filenameAbsPath := configDirAbsPath.Join(c.basename, chezmoi.NewRelPath(c.basename.String()+"."+extension))
 			if _, err := system.Stat(filenameAbsPath); err == nil {
 				filenameAbsPaths[filenameAbsPath] = struct{}{}
@@ -526,8 +524,7 @@ func (c *dirCheck) Run(system chezmoi.System, homeDirAbsPath chezmoi.AbsPath) (c
 		if dirEntry.Name() != ".git" {
 			continue
 		}
-		//nolint:gosec
-		cmd := exec.Command("git", "-C", c.dirname.String(), "status", "--porcelain=v2")
+		cmd := exec.Command("git", "-C", c.dirname.String(), "status", "--porcelain=v2") //nolint:gosec
 		output, err := cmd.Output()
 		if err != nil {
 			gitStatus = gitStatusError
