@@ -221,11 +221,13 @@ func (c *Config) runStateResetCmd(cmd *cobra.Command, args []string) error {
 		return err
 	}
 	if !c.force {
-		switch choice, err := c.promptChoice(fmt.Sprintf("Remove %s", persistentStateFileAbsPath), []string{"yes", "no"}); {
+		switch choice, err := c.promptChoice(fmt.Sprintf("Remove %s", persistentStateFileAbsPath), choicesYesNoQuit); {
 		case err != nil:
 			return err
 		case choice == "yes":
 		case choice == "no":
+			fallthrough
+		case choice == "quit":
 			return nil
 		}
 	}
