@@ -32,6 +32,7 @@ import (
 const (
 	upgradeMethodBrewUpgrade       = "brew-upgrade"
 	upgradeMethodReplaceExecutable = "replace-executable"
+	upgradeMethodSnapRefresh       = "snap-refresh"
 	upgradeMethodUpgradePackage    = "upgrade-package"
 	upgradeMethodSudoPrefix        = "sudo-"
 
@@ -172,6 +173,10 @@ func (c *Config) runUpgradeCmd(cmd *cobra.Command, args []string) error {
 		if err := c.replaceExecutable(ctx, executableAbsPath, version, rr); err != nil {
 			return err
 		}
+	case upgradeMethodSnapRefresh:
+		if err := c.snapRefresh(); err != nil {
+			return err
+		}
 	case upgradeMethodUpgradePackage:
 		useSudo := false
 		if err := c.upgradePackage(ctx, version, rr, useSudo); err != nil {
@@ -247,7 +252,7 @@ func (c *Config) downloadURL(ctx context.Context, url string) ([]byte, error) {
 	}
 	if resp.StatusCode != http.StatusOK {
 		_ = resp.Body.Close()
-		return nil, fmt.Errorf("%s: got a non-200 OK response: %d %s", url, resp.StatusCode, resp.Status)
+		return nil, fmt.Errorf("%s: %s", url, resp.Status)
 	}
 	data, err := io.ReadAll(resp.Body)
 	if err != nil {
@@ -371,6 +376,10 @@ func (c *Config) replaceExecutable(
 	return
 }
 
+func (c *Config) snapRefresh() error {
+	return c.run(chezmoi.EmptyAbsPath, "snap", []string{"refresh", c.upgrade.repo})
+}
+
 func (c *Config) upgradePackage(
 	ctx context.Context, version *semver.Version, rr *github.RepositoryRelease, useSudo bool,
 ) error {
@@ -499,6 +508,9 @@ func getUpgradeMethod(fileSystem vfs.Stater, executableAbsPath chezmoi.AbsPath)
 	case "freebsd":
 		return upgradeMethodReplaceExecutable, nil
 	case "linux":
+		if ok, _ := vfs.Contains(fileSystem, executableAbsPath.String(), "/snap"); ok {
+			return upgradeMethodSnapRefresh, nil
+		}
 		fileInfo, err := fileSystem.Stat(executableAbsPath.String())
 		if err != nil {
 			return "", err
